lm.component('pageFiles', {
  templateUrl: 'comp/page/pageFiles.' + lmCacheTag + 'html',
  bindings: {
  },
  $canActivate: function($nextInstruction, $prevInstruction, serviceLimboMedia) {
	  return serviceLimboMedia.isAuthenticated();
  },
  controller: function($scope, $http, $timeout, $rootRouter, serviceNotification, serviceLimboMedia, serviceMediaPlayer, serviceUpload, serviceClipboard, serviceModal, serviceGlobalEvents) {
		var ctrl = this;
	  
		/* ========== */
		/* = SEARCH = */
		/* ========== */	
		ctrl.search = {active: false, value: ''};
		ctrl.searchClear = function() {
			ctrl.search.active = false;
			ctrl.search.value = '';
		}
		ctrl.onSearchChanged = function() {
			ctrl.folderLoad();
		};

		/* ========== */
		/* = FILTER = */
		/* ========== */
		ctrl.filters = [
			{id: 'ALL',   i18n: 'FFILTER_NONE',  icon: 'icon-file'		, order: 'NAME'},
			{id: 'IMAGE', i18n: 'FFILTER_IMAGE', icon: 'icon-file-image', order: 'NAME'},
			{id: 'AUDIO', i18n: 'FFILTER_AUDIO', icon: 'icon-file-audio', order: 'TITLE'},
			{id: 'VIDEO', i18n: 'FFILTER_VIDEO', icon: 'icon-file-video', order: 'NAME'}
		];
		
		ctrl.filter = ctrl.filters[0];
		
		ctrl.toggleFilter = function() {
			ctrl.filter = lmBase.arrayGetNext(ctrl.filters, ctrl.filter);
			ctrl.predicate = ctrl.filter.order;
			ctrl.settings.filter = ctrl.filter.id;
			ctrl.saveSettings();
			ctrl.filesLoad();
		}


		/* ======== */
		/* = VIEW = */
		/* ======== */
		ctrl.views = [
			{id: 'LIST',    i18n: 'LIST',    icon: 'icon-list'},
			{id: 'GALLERY', i18n: 'GALLERY', icon: 'icon-grid'}
		];
		
		ctrl.view = ctrl.views[0];

		ctrl.toggleView = function() {
			ctrl.view = lmBase.arrayGetNext(ctrl.views, ctrl.view);
			ctrl.settings.view = ctrl.view.id;
			ctrl.saveSettings();
			ctrl.filesLoad();
		}
		
		ctrl.listClazz = 'vM';
		ctrl.listThumb = 'TXS_QC';
		ctrl.listShowThumb = true;
		
		ctrl.galleryClazz = 'vM';
		ctrl.galleryThumb = 'TL_H';

		ctrl.refreshDesign = function() {
			if('SMALL' == ctrl.settings.list.size) {
				ctrl.listClazz = 'vS';
				ctrl.listThumb = 'NONE';
				ctrl.listShowThumb = false;
			}
			else if('LARGE' == ctrl.settings.list.size) {
				ctrl.listClazz = 'vL';
				ctrl.listThumb = 'TS_QC';
				ctrl.listShowThumb = true;
			}
			else {
				ctrl.listClazz = 'vM';
				ctrl.listThumb = 'TXS_QC';
				ctrl.listShowThumb = true;
			}
	
			if('SMALL' == ctrl.settings.gallery.size) {
				ctrl.galleryClazz = 'vS';
				ctrl.galleryThumb = 'TM_H';
			}
			else if('LARGE' == ctrl.settings.gallery.size) {
				ctrl.galleryClazz = 'vL';
				ctrl.galleryThumb = 'TXL_H';
			}
			else {
				ctrl.galleryClazz = 'vM';
				ctrl.galleryThumb = 'TL_H';
			}
		};
		
		/* ========= */
		/* = ORDER = */
		/* ========= */
		ctrl.predicate = 'NAME';
		ctrl.dir = 'asc';	
		
		ctrl.changeOrder = function(valueOrder) {
			if(ctrl.predicate == valueOrder) {
				ctrl.dir = ctrl.dir == 'asc' ? 'desc' : 'asc';
			}
			else {
				ctrl.dir = 'asc';
				ctrl.predicate = valueOrder;
			}
			ctrl.filesLoad();
		};
		
		/* =============================== */
		/* = CURRENT FOLDER / SUBFOLDERS = */
		/* =============================== */
		
		ctrl.folderId = 0;				/* folder id as given via url */
		
		ctrl.folder = null;				/* current folder */
		ctrl.folderStack = [];			/* folder stack (for breadcrumb) */
		ctrl.folders = [];				/* list of subfolders */
		
		ctrl.folderLoad = function() {
			ctrl.folders = [];

			if(ctrl.search.active) {
				var par = {folder: ctrl.folderId, fr: 0, mr: 100, s: ctrl.search.value};
				$http.get(lmBase.url + 'f/folder/search',{params : par}).then(
					function success(resp) {
						ctrl.folders = resp.data;
						ctrl.filesLoad();
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
			else {
				$http.get(lmBase.url + 'f/folder/' + ctrl.folderId + '/stacklist').then(
					function success(resp) {
						ctrl.folderStack = resp.data.stack;
						ctrl.folder = resp.data.stack.length == 0 ? null : resp.data.stack[resp.data.stack.length - 1];
						ctrl.folders = resp.data.items;
						ctrl.filesLoad();
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
		};
		
		/* ================= */
		/* = CUT AND PASTE = */
		/* ================= */
		
		ctrl.clipboard = serviceClipboard;
		
		/* true = copy, false = move */
		ctrl.clipOp = function(docopy) {
			var it = serviceClipboard.next();
			
			if(it == null) {
				serviceClipboard.working = false;
				ctrl.folderLoad();
				return;
			}
			serviceClipboard.working = true;
			
			var itval = it.value;
			
			var req = {target: ctrl.folder.id};
			var earl = lmBase.url + (itval.type == null ? 'f/folder/' : 'f/file/') + itval.id + (docopy ? '/copy' : '/move');
			$http.post(earl,req).then(
				function success(resp) {
					serviceClipboard.remove(it);
					ctrl.clipOp(docopy);
				},
				function error(resp) {
					serviceLimboMedia.handleHttpErrorNew(resp);
					serviceLimboMedia.httpErrorToMsg(resp, function(msg) {
						it.error = msg;
						ctrl.clipOp(docopy);
					});
				}
			);
		}
		
		/* ============== */
		/* = FOLDER OPS = */
		/* ============== */	
		
		ctrl.folderDelete = function(valueItem) {
			serviceModal.show('lm-dialog-folder-delete',{folder: valueItem}).then(
				function() {
					var idx = ctrl.folders.indexOf(valueItem);
					if(idx != -1) ctrl.folders.splice(idx,1);
				},
				function() {}
			);
		};
		
		ctrl.folderShares = function(valueItem) {
			serviceModal.show('lm-dialog-share',{user: valueItem.x.user, url: lmBase.url + 'f/folder/' + valueItem.id + '/shares'}).then(
				  function() { ctrl.folderLoad(); },
				  function() { }
			);
		};
		
		ctrl.folderTokens = function(valueItem) {
			serviceModal.show('lm-dialog-folder-tokens',{id: valueItem.id}).then(function() {}, function() {});
		};
		
		ctrl.folderEdit = function(valueItem) {
			serviceModal.show('lm-dialog-rename',{name: valueItem.name, url: lmBase.url + 'f/folder/' + valueItem.id}).then(
				  function(result) {
					var idx = ctrl.folders.indexOf(valueItem);
					if(result && idx != -1) {
						ctrl.folders[idx] = result;
					}				  
				  },
				  function() {}
			);
		};	
		
		ctrl.folderSynchronize = function(valueFolder) {
			$http.post(lmBase.url + 'f/folder/' + valueFolder.id + '/sync').then(
				function success(resp) {
					serviceNotification.ok('MSG_JOB_STARTED');
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		
		ctrl.folderMount = function() {
			serviceModal.show('lm-dialog-folder-mount').then(
				function() { ctrl.folderLoad(); },
				function() {}
			);
		};
		
		
		/* ========= */
		/* = FILES = */
		/* ========= */
		ctrl.files = [];

		/* no of items in view */
		ctrl.fileLimit = 0;
		
		ctrl.filesClear = function() {
			ctrl.files = [];
			ctrl.fileLimit = ctrl.settings.pageSize;
		};	
		
		ctrl.filesLoad = function() {
			ctrl.filesClear();
			
			var par = {
				order: ctrl.predicate,
				dir: ctrl.dir,
				details: false
			};
			
			if('IMAGE' == ctrl.filter.id) par.fileType = 'IMAGE';
			if('AUDIO' == ctrl.filter.id) par.fileType = 'AUDIO';
			if('VIDEO' == ctrl.filter.id) par.fileType = 'VIDEO';
			
			if(ctrl.search.active) {
				par.mr = 500;
				par.s = ctrl.search.value;
				if(ctrl.folder != null) {
					par.folder = ctrl.folder.id;
				}
				
				$http.get(lmBase.url + 'f/files/search',{params : par}).then(
					function success(resp) {
						for(x in resp.data) {
							var it = resp.data[x];
							lmBase.pimpLmFile(it);
							ctrl.files.push(it);
						}
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}
			else {
				if(ctrl.folder == null) return;
				
				par.folder = ctrl.folder.id;
				
				$http.get(lmBase.url + 'f/files/list',{params : par}).then(
					function success(resp) {
						for(x in resp.data) {
							var it = resp.data[x];
							lmBase.pimpLmFile(it);
							ctrl.files.push(it);
						}
					},
					serviceLimboMedia.handleHttpErrorNew
				);
			}			
		};
		
		ctrl.filesShowMore = function() {
			ctrl.fileLimit += ctrl.settings.pageSize;
		};

		ctrl.filesShowAll = function() {
			ctrl.fileLimit = null;
		};

		ctrl.deleteFile = function(valueItem) {
			$http.delete(lmBase.url + 'f/file/' + valueItem.id).then(
				function success(resp) {
					var idx = ctrl.files.indexOf(valueItem);
					if(idx != -1) ctrl.files.splice(idx,1);
				},
				serviceLimboMedia.handleHttpErrorNew
			);				
		};
		
		ctrl.selectionToggle = function() {
			if(ctrl.files.length > 0) {
				var targetState = !ctrl.files[0].client.select;
				for(x in ctrl.files) {
					ctrl.files[x].client.select = targetState;
				}
			}
		};
		
		ctrl.selectionDownload = function() {
		    var ids = new Array();
		    for(x in ctrl.files) {
				if(ctrl.files[x].client.select) {
					ids.push(ctrl.files[x]);
				}
			}
			
		    if(ids.length > 0) {
			    var ifrm = document.getElementById('downloadframe');
		    	ifrm.src = lmBase.url + 'f/streamZip/LimboDownloadFiles.zip?ids=' + lmBase.arrayToCSV(ids);
		    }
		};

		ctrl.folderDownload = function(valueFolder) {
		    var filename = 'LimboDownload_' + encodeURIComponent(valueFolder.name) + ".zip";
		    var ifrm = document.getElementById('downloadframe');
	    	ifrm.src = lmBase.url + 'f/folder/' + valueFolder.id + '/zip/' + filename;
		};

		ctrl.folderMap = function(valueFolder) {
			$rootRouter.navigate(['AppMapFolder', {folderId: valueFolder.id}]);
		};
		
		ctrl.getSelectedFiles = function(typefilter) {
			var result = new Array();
			
			for(x in ctrl.files) {
				var it = ctrl.files[x];
				if(it.client.select && (typefilter == null || typefilter(it.type))) {
					result.push(it);
				}
			}
			
			return result;
		};
		
		/* ============ */
		/* = FILE OPS = */
		/* ============ */	
		
		ctrl.fileEdit = function(valueItem) {
			serviceModal.show('lm-dialog-file-edit',{id: valueItem.id}).then(
				function(result) {
					var idx = ctrl.files.indexOf(valueItem);
					if(result && idx != -1) {
						lmBase.pimpLmFile(result);
						ctrl.files[idx] = result;
					}
				},
				function() {}
			);
		};
		
		ctrl.defaultFileAction = function(valueFile) {
			if('AUDIO' == valueFile.type || 'VIDEO' == valueFile.type) serviceMediaPlayer.addPlayFile(valueFile);
			else if(lmBase.isFilePdf(valueFile)) $rootRouter.navigate(['AppPdf', {fileId: valueFile.id}]);
			else if('IMAGE' == valueFile.type) ctrl.imageShow(valueFile);
			else if(lmBase.isFileText(valueFile)) $rootRouter.navigate(['AppEditor', {fileId: valueFile.id}]);
			else ctrl.downloadFile(valueFile);
		};
		
		ctrl.downloadFile = function(item) {
			var ifrm = document.getElementById('downloadframe');
			ifrm.src = lmBase.url + 'f/stream/' + item.id + '/' + item.name;
		};
		
		ctrl.mediaPlay = function(valueItem) {
			serviceMediaPlayer.playFile(valueItem);
		};
		
		ctrl.mediaAdd = function(valueItem) {
			serviceMediaPlayer.addFile(valueItem);
		};

		ctrl.mediaAddPlay = function(valueItem) {
			serviceMediaPlayer.addPlayFile(valueItem);
		};
		
		ctrl.mediaPlayAll = function() {
			serviceMediaPlayer.playFiles(ctrl.getSelectedFiles(function(type) {return 'AUDIO' == type || 'VIDEO' == type}));
		};
		
		ctrl.mediaAddAll = function() {
			serviceMediaPlayer.addFiles(ctrl.getSelectedFiles(function(type) {return 'AUDIO' == type || 'VIDEO' == type}));
		};
		
		ctrl.selectionClip = function() {
			var result = ctrl.getSelectedFiles();
			for(x in result) ctrl.clipboard.clip(result[x]);
		};

		ctrl.selectionDelete = function() {
			var result = ctrl.getSelectedFiles();
			for(x in result) ctrl.deleteFile(result[x]);
		};
		
		ctrl.mediaExternal = function(valueItem) {
			serviceModal.show('lm-dialog-file-links',{id: valueItem.id}).then(function() {}, function() {});
		};

		ctrl.fileEditor = function(valueItem) {
			$rootRouter.navigate(['AppEditor', {fileId: valueItem.id}]);
		};	

		ctrl.fileMap = function(valueItem) {
			$rootRouter.navigate(['AppMapFile', {fileId: valueItem.id}]);
		};

		ctrl.filePdf = function(valueItem) {
			$rootRouter.navigate(['AppPdf', {fileId: valueItem.id}]);
		};	

		ctrl.openFolder = function(valueItem) {
			$http.get(lmBase.url + 'f/file/' + valueItem.id + '/folder').then(
				function success(resp) {
					ctrl.searchClear();
					$rootRouter.navigate(['FilesAndMedia', {id: resp.data.id}]);
				}, serviceLimboMedia.handleHttpErrorNew
			);			
		};	
		
		/* ============= */
		/* = NEW STUFF = */
		/* ============= */
		ctrl.uploadFiles = new Array();
		
		/* Functions for direct JS calls (non-angular) */	
		ctrl.onFileSelect = function(valueFiles) {
			$scope.$apply(function() {
				if(!ctrl.folder) {
					serviceNotification.error('MSG_NO_ROOT');
					return;
				}
				
				if(!ctrl.folder.x.write) {
					serviceNotification.error('MSG_NO_RIGHTS');
					return;
				}

				var files = new Array();
			
				for(x in valueFiles) {
					var el = valueFiles[x];
					if(el instanceof File) {
						files.push(el);
					}
				};
				/* let upload service manage this */
				var par = new Object();
				par.folder = ctrl.folder == null ? 0 : ctrl.folder.id;
				serviceUpload.uploadFiles(files,lmBase.url + 'f/file', false, par);
			});
		};
		
		ctrl.onClassicUpload = function(element) {
			ctrl.onFileSelect(element.files);
			 // Reset input to fire onChange even if user selects same file.
			element.value = "";
		};
		

		/* ====================== */
		/* = DragDrop callbacks = */
		/* ====================== */
		
		ctrl.onDdCheck = function() {
			if(!ctrl.folder) return {allowed: false, msg: 'MSG_NO_ROOT'};
			if(!ctrl.folder.x.write) return {allowed: false, msg: 'MSG_NO_RIGHTS'};
			return {allowed: true, msg: 'MSG_UPLOAD_DROP'};
		};

		ctrl.onDdDrop = function(valueDataTransfer) {
			ctrl.onFileSelect(valueDataTransfer.files);
		};

		/* ======================================= */
		/* = (De)register on events and services = */
		/* ======================================= */
		
		serviceUpload.registerListener(ctrl.folderLoad);
		
		/* Listen to global filedrop */
		serviceGlobalEvents.registerListenerDataTransfer(ctrl);
		
		/* destructor */
		$scope.$on('$destroy', function iVeBeenDismissed() {
			serviceUpload.registerListener(null);
			serviceGlobalEvents.registerListenerDataTransfer(null);
		});
		
		
		ctrl.folderCreate = new Object();
		ctrl.folderCreate.name = '';		/* new folders name */

		ctrl.folderCreate.create = function() {
			var req = {name: ctrl.folderCreate.name };
			req.parent = ctrl.folder == null ? 0 : ctrl.folder.id;
			
			$http.post(lmBase.url + 'f/folder',req).then(
				function success(resp) {
					ctrl.folderLoad();
					ctrl.folderCreate.name = '';
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		

		/* ========= */
		/* = IMAGE = */
		/* ========= */
		ctrl.imageCurrent = null;

		ctrl.imageShow = function(valueFile) {
			ctrl.imageCurrent = valueFile;
		};
		
		ctrl.imageClose = function() {
			ctrl.imageCurrent = null;
		};

		/* ============ */
		/* = SETTINGS = */
		/* ============ */
		ctrl.settings = new Object();
			
		ctrl.toggleConfig = function() {
			serviceModal.show('lm-dialog-file-settings').then(
				function() { ctrl.refreshDesign(); ctrl.folderLoad(); },
				function() { ctrl.refreshDesign(); ctrl.folderLoad(); }
			);
		};
		
		ctrl.saveSettings = function() {
			serviceLimboMedia.saveRemoteSettings();
		};	
		
		ctrl.loadSettings = function() {
			ctrl.settings = serviceLimboMedia.getRemoteSettings('files');
			lmBase.defaultSettingsFiles(ctrl.settings);

			ctrl.view   = lmBase.arrayGetByIdOrFirst(ctrl.views, ctrl.settings.view);
			ctrl.filter = lmBase.arrayGetByIdOrFirst(ctrl.filters, ctrl.settings.filter);  
 
			/* =========== */
			/* = STARTUP = */
			/* =========== */
			ctrl.refreshDesign();
			ctrl.folderLoad();
		}
		
		ctrl.$routerOnActivate = function(next) {
		    var id = next.params.id;
		    if(id) {
		    	ctrl.folderId = id;
		    }
		    ctrl.loadSettings();
		};
	  
  }
});